@@ -12,9 +12,9 @@
 
    Copyright 2022, HSPyLib team
 """
-from abc import ABC
 from typing import Generic, List, Optional, TypeVar
 
+from hspylib.core.metaclass.singleton import AbstractSingleton
 from hspylib.core.namespace import Namespace
 
 from datasource.crud_entity import CrudEntity
@@ -25,7 +25,7 @@ E = TypeVar('E', bound=CrudEntity)
 R = TypeVar('R', bound=CrudRepository)
 
 
-class CrudService(Generic[R, E], ABC):
+class CrudService(Generic[R, E], metaclass=AbstractSingleton):
 
     def __init__(self, repository: R):
         self._repository = repository
