@@ -14,11 +14,12 @@
 """
 import contextlib
 import logging as log
-from typing import List, Optional, Set, Tuple, TypeVar
+from typing import Generic, List, Optional, Set, Tuple, TypeVar
 
 import psycopg2
 from hspylib.core.exception.exceptions import DatabaseConnectionError
-from hspylib.core.tools.namespace import Namespace
+from hspylib.core.metaclass.singleton import AbstractSingleton
+from hspylib.core.namespace import Namespace
 from hspylib.core.tools.text_tools import quote
 from psycopg2 import DatabaseError, OperationalError, ProgrammingError
 from retry import retry
@@ -28,10 +29,10 @@ from datasource.db_configuration import DBConfiguration
 from datasource.db_repository import Connection, Cursor, DBRepository, ResultSet, Session
 from datasource.identity import Identity
 
-T = TypeVar('T', bound=CrudEntity)
+E = TypeVar('E', bound=CrudEntity)
 
 
-class PostgresRepository(DBRepository[T, DBConfiguration]):
+class PostgresRepository(Generic[E], DBRepository[E, DBConfiguration], metaclass=AbstractSingleton):
     """Implementation of a data access layer for a postgres persistence store."""
 
     def __init__(self, config: DBConfiguration):
@@ -90,14 +91,14 @@ class PostgresRepository(DBRepository[T, DBConfiguration]):
         sql = f"DELETE FROM {self.table_name()} WHERE " + ' AND '.join(clauses)
         self.execute(sql)
 
-    def delete_all(self, entities: List[T]) -> None:
+    def delete_all(self, entities: List[E]) -> None:
         values, s = [], entities[0]
         list(map(lambda e: values.append(str(e.values)), entities))
         sql = f"DELETE FROM " \
               f"{self.table_name()} WHERE ({s.as_columns()}) IN ({', '.join(values)}) "
         self.execute(sql)
 
-    def save(self, entity: T) -> None:
+    def save(self, entity: E) -> None:
         columns, ids = entity.as_columns(), set(entity.identity.attributes)
         sql = f"INSERT INTO " \
               f"{self.table_name()} ({columns}) VALUES {entity.values} " \
@@ -105,7 +106,7 @@ class PostgresRepository(DBRepository[T, DBConfiguration]):
               f"DO UPDATE SET {entity.as_column_set(prefix='EXCLUDED.', exclude=ids)}"
         self.execute(sql)
 
-    def save_all(self, entities: List[T]) -> None:
+    def save_all(self, entities: List[E]) -> None:
         values, sample = [], entities[0]
         columns, ids = sample.as_columns(), set(sample.identity.attributes)
         list(map(lambda e: values.append(str(e.values)), entities))
@@ -120,7 +121,7 @@ class PostgresRepository(DBRepository[T, DBConfiguration]):
         fields: Optional[Set[str]] = None,
         filters: Optional[Namespace] = None,
         order_bys: Optional[List[str]] = None,
-        limit: int = 500, offset: int = 0) -> List[T]:
+        limit: int = 500, offset: int = 0) -> List[E]:
 
         fields = '*' if not fields else ', '.join(fields)
         clauses = list(filter(None, filters.values)) if filters else None
@@ -135,7 +136,7 @@ class PostgresRepository(DBRepository[T, DBConfiguration]):
     def find_by_id(
         self,
         entity_id: Identity,
-        fields: Optional[Set[str]] = None) -> Optional[T]:
+        fields: Optional[Set[str]] = None) -> Optional[E]:
 
         fields = '*' if not fields else ', '.join(fields)
         clauses = [f"{k} = {quote(v)}" for k, v in zip(entity_id.attributes, entity_id.values)]
